/*****************************************************************************
 *
 * FILE:	param.c
 * DESCRIPTION:	parameter manipulator
 * DATE:	Sat, Mar 16 2002
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2002 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: param.c,v 1.5 2002/07/21 12:51:28 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cockatrice.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
#define	PARAMETER_TABLE_SIZE	13

typedef struct _param {
  String		name;
  VList *		vlist;	/* value list */
  struct _param *	next;
} Param;


/*****************************************************************************
 *
 *	definition of local functions
 *
 *****************************************************************************/
static Param *	lookup(const String);


/*****************************************************************************
 *
 *	definition of local variables
 *
 *****************************************************************************/
static Param *	param[PARAMETER_TABLE_SIZE];


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * FUNCTION NAME:	paramInit
 * DESCRIPTION:		initialize a parameter table
 * ARGUMENTS:		none
 * RETURN VALUE:	none
 */
void
paramInit(void)
{
  int	i;

  for (i = 0; i < PARAMETER_TABLE_SIZE; i++) {
    param[i] = NULL;
  }
}

/*
 * FUNCTION NAME:	paramRelease
 * DESCRIPTION:		release memory area of parameter table
 * ARGUMENTS:		none
 * RETURN VALUE:	none
 */
void
paramRelease(void)
{
  int	i;

  for (i = 0; i < PARAMETER_TABLE_SIZE; i++) {
    register Param *	pp;
    register Param *	tmp;

    if (param[i] == NULL) {
      continue;
    }

    for (pp = param[i]; pp != NULL; pp = tmp) {
      tmp = pp->next;
      if (pp->name != NULL) {
	free((String)pp->name);
      }
      if (pp->vlist != NULL) {
	register VList *	vp;
	register VList *	vtmp;

	for (vp = pp->vlist; vp != NULL; vp = vtmp) {
	  vtmp = vp->next;
	  free((String)vp->value);
	  free((VList *)vp);
	}
	pp->vlist = NULL;
      }
      free((Param *)pp);
    }

    param[i] = NULL;
  }
}

/*
 * FUNCTION NAME:	paramPut
 * DESCRIPTION:		store CGI parameter into the table
 * ARGUMENTS:		name, value
 * RETURN VALUE:	Successful: 0, otherwise: -1
 */
int
paramPut(name, value)
	const String	name;
	const String	value;
{
  VList *	vp;	/* value list pointer */
  Param *	pp;	/* new parameter pointer */
  unsigned int	h;	/* hash value */

  if (name == NULL) {
    return -1;
  }

  vp = (VList *)calloc(1, sizeof(VList));
  if (vp == NULL) {
    return -1;
  }
#if	0
  vp->value = (value != NULL && *value != EOL) ? strdup(value) : NULL;
#else
  vp->value = value != NULL ? strdup(value) : NULL;
#endif
  vp->next  = NULL;

  pp = lookup(name);
  if (pp == NULL) {	/* first register */
    pp = (Param *)calloc(1, sizeof(Param));
    if (pp == NULL) {
      return -1;
    }

    h		= hash(name, PARAMETER_TABLE_SIZE);
    pp->name	= strdup(name);
    pp->vlist	= vp;
    pp->next	= param[h];
    param[h]	= pp;
  }
  else {		/* already registered name */
    vp->next	= pp->vlist;
    pp->vlist	= vp;
  }

  return 0;
}

/*
 * FUNCTION NAME:	paramGet
 * DESCRIPTION:		get CGI value from the table
 * ARGUMENTS:		name
 * RETURN VALUE:	value
 */
String
paramGet(name)
	const String	name;
{
  register Param *	pp;

  pp = lookup(name);
  if (pp != NULL) {
    return pp->vlist->value;
  }

  return NULL;
}

/*
 * FUNCTION NAME:	paramsGet
 * DESCRIPTION:		get CGI values from the table
 * ARGUMENTS:		name
 * RETURN VALUE:	values
 */
VList *
paramsGet(name)
	const String	name;
{
  register Param *	pp;

  pp = lookup(name);
  if (pp != NULL) {
    return pp->vlist;
  }

  return NULL;
}

/*
 * FUNCTION NAME:	lookup
 * DESCRIPTION:		look up a parameter name
 * ARGUMENTS:		name
 * RETURN VALUE:	Successful: parameter pointer, otherwise: NULL
 */
static Param *
lookup(name)
	const String	name;
{
  register Param *	pp;	/* working pointer */
  unsigned int		h;	/* hash value */

  if (name != NULL) {
    h = hash(name, PARAMETER_TABLE_SIZE);
    for (pp = param[h]; pp != NULL; pp = pp->next) {
      if (pp->name != NULL && strcmp(name, pp->name) == 0) {
	return pp;
      }
    }
  }

  return NULL;
}

