/*****************************************************************************
 *
 * FILE:	util.c
 * DESCRIPTION:	utilities for simple CGI library
 * DATE:	Sat, Mar 16 2002
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2002 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: util.c,v 1.4 2002/06/16 14:35:00 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <time.h>
#if	TIME_WITH_SYS_TIME
#include <sys/time.h>
#endif	/* TIME_WITH_SYS_TIME */
#ifndef	HAVE_MKSTEMP
#include <fcntl.h>
#endif	/* HAVE_MKSTEMP */
#include <md5.h>
#include <sha.h>
#include "cockatrice.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
#ifndef	MAX_URI_LEN
#define	MAX_URI_LEN	1024
#endif	/* MAX_URI_LEN */


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	chomp
 * Description:		remove  CRLF, CR or LF from string end
 * Arguments:		string
 * Return Value:	string
 */
String
chomp(str)
	String	str;
{
  String	p;
  size_t	len;

  if (str == NULL) {
    return str;
  }

  len = strlen(str);
  p   = &str[len - 1];
  if (*p == LF) {	/* remove LF */
    *p-- = EOL;
  }
  if (*p == CR) {	/* remove CR */
    *p-- = EOL;
  }

  return str;
}

/*
 * FUNCTION NAME:	hash
 * DESCRIPTION:		calculates a hash value of string
 * ARGUMENTS:		string, table size
 * RETURN VALUE:	hash value
 */
unsigned int
hash(str, size)
	String	str;
	size_t	size;
{
  register String	p;
  register unsigned int	h;

  p = str;
  for (h = *p++; *p != EOL; p++) {
    h += *p;
  }

  return h % size;
}


/*****************************************************************************
 *****************************************************************************/

/*
 * FUNCTION NAME:	escapeURI
 * DESCRIPTION:		encode URI by escaped characters
 * ARGUMENTS:		string
 * RETURN VALUE:	escaped string
 * Reference:		RFC 2396
 *			Uniform Resource Identifiers (URI): Generic Syntax
 */
String
escapeURI(s)
	String	s;
{
  int		i;
  int		j;
  size_t	len;
  String	enc;

  if (s == NULL) {
    return NULL;
  }

  len = strlen(s);
  enc = (String)calloc(len * 3 + 1, sizeof(char));
  if (enc == NULL) {
    return NULL;
  }

  for (i = 0, j = 0; i < len; i++) {
    if ((s[i] & 0x80) == 0x80 ||	/* Japanese EUC code */
	s[i] == ';' || s[i] == '/' || s[i] == '?' || s[i] == ':' ||
	s[i] == '@' || s[i] == '&' || s[i] == '=' || s[i] == '+' ||
	s[i] == '$' || s[i] == ',') {
      char	b1 = (s[i] >> 4) & 0x0f;
      char	b2 = s[i] & 0x0f;
      enc[j++] = '%';
      enc[j++] = b1 < 10 ? b1 + '0' : b1 - 10 + 'A';
      enc[j++] = b2 < 10 ? b2 + '0' : b2 - 10 + 'A';
    }
    else {
      enc[j++] = s[i];
    }
  }

  /* squeeze */
  if (j < len * 3 + 1) {
    enc = (String)realloc(enc, (size_t)j);
  }

  return enc;
}

/*
 * FUNCTION NAME:	unescapeURI
 * DESCRIPTION:		decode an escaped character
 * ARGUMENTS:		string
 * RETURN VALUE:	unescaped string
 * REFERENCE:		RFC 2396
 *			Uniform Resource Identifiers (URI): Generic Syntax
 */
String
unescapeURI(s)
	String	s;
{
  char			tmp[MAX_URI_LEN];
  register String	os  = tmp;
  register String	ret = s;
  char			c, c1;

  /* copy original requst URI into working buffer */
  memset(tmp, 0, sizeof(tmp));
  memcpy(tmp, s, sizeof(tmp));

  while ((c = *os) != EOL) {
    /* escaped URI */
    if (c == '%') {
      os++;
      if ((c = *os++) && (c1 = *os++)) {
	*s++ =	((c  >= 'A' ? ((c  & 0xdf) - 'A') + 10 : c  - '0') * 16) +
		((c1 >= 'A' ? ((c1 & 0xdf) - 'A') + 10 : c1 - '0'));
	os--;	/* back one word */
      }
      else {
        return ret;
      }
    }
    /* escaped white space */
    else if (c == '+') {
      *s++ = ' ';
    }
    /* normal characters */
    else {
      *s++ = c;
    }
    os++;
  }
  *s = EOL;

  return ret;
}

/*
 * FUNCTION NAME:	digestString
 * DESCRIPTION:		digest string
 * ARGUMENTS:		string, digest_t
 * RETURN VALUE:	Successful: Digest, otherwise: NULL
 */
#ifndef	MD5_DIGEST_LENGTH
#define	MD5_DIGEST_LENGTH	16
#endif	/* MD5_DIGEST_LENGTH */
String
digestString(str, type)
	const String	str;
	digest_t	type;
{
  if (str == NULL) {
    return NULL;
  }

  switch (type) {
    case DIGEST_MD5: {
	MD5_CTX			md5_ctx;
	unsigned char		md5[MD5_DIGEST_LENGTH];
	static unsigned char	md5_hex[MD5_DIGEST_LENGTH*2+1];
	size_t			len = strlen(str);
	String			buf = strdup(str);
	register int		i;

	MD5Init(&md5_ctx);
	MD5Update(&md5_ctx, buf, len);
	MD5Final(md5, &md5_ctx);

	memset(md5_hex, 0, sizeof(md5_hex));
	for (i = 0; i < MD5_DIGEST_LENGTH; i++) {
	  snprintf(md5_hex + i * 2, sizeof(md5_hex), "%02x", md5[i]);
	}
	free((String)buf);

	return md5_hex;
      }
      break;
    case DIGEST_SHA1: {
	SHA_CTX			sha_ctx;
	unsigned char		sha[SHA_DIGEST_LENGTH];
	static unsigned char	sha_hex[SHA_DIGEST_LENGTH*2+1];
	size_t			len = strlen(str);
	String			buf = strdup(str);
	register int		i;

	SHA1_Init(&sha_ctx);
	SHA1_Update(&sha_ctx, buf, len);
	SHA1_Final(sha, &sha_ctx);

	memset(sha_hex, 0, sizeof(sha_hex));
	for (i = 0; i < SHA_DIGEST_LENGTH; i++) {
	  snprintf(sha_hex + i * 2, sizeof(sha_hex), "%02x", sha[i]);
	}
	free((String)buf);

	return sha_hex;
      }
      break;
    default:
      break;
  }

  return NULL;
}


/*****************************************************************************
 *****************************************************************************/

#ifndef	HAVE_MKSTEMP
/*
 * FUNCTION NAME:	mkstemp
 * DESCRIPTION:		create temporary file and return its file descriptor
 * ARGUMENTS:		template path
 * RETURN VALUE:	file descriptor
 */
int
mkstemp(path)
	String	path;
{
  String	p = mktemp(path);

  if (p == NULL) {
    return -1;
  }

  return open(path, O_CREAT|O_EXCL|O_RDWR, 0600);
}
#endif	/* HAVE_MKSTEMP */
